package com.yizhi.drools.application.controller.manage;

import com.alibaba.fastjson.JSONReader;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.drools.application.constant.DroolsConstants;
import com.yizhi.drools.application.domain.RecommendRule;
import com.yizhi.drools.application.domain.RecommendRuleItem;
import com.yizhi.drools.application.feign.RecommendClient;
import com.yizhi.drools.application.service.IRecommendRuleItemService;
import com.yizhi.drools.application.service.IRecommendRuleService;
import com.yizhi.drools.application.service.IRecommendStrategyRuleService;
import com.yizhi.drools.application.service.IRecommendStrategyService;
import com.yizhi.drools.application.util.ContextUtil;
import com.yizhi.drools.application.util.ResponseUtil;
import com.yizhi.drools.application.vo.drools.*;
import com.yizhi.drools.application.vo.manage.*;
import com.yizhi.util.application.domain.Response;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author shengchenglong123
 * @since 2019-11-27
 */
@Slf4j
@RestController
@RequestMapping("/manage/recommendRule")
@Api(tags = "推荐规则 - 管理端接口")
public class RecommendRuleController {

    @Autowired
    private IRecommendRuleService ruleService;
    @Autowired
    private IRecommendRuleItemService ruleItemService;
    @Autowired
    private IRecommendStrategyRuleService strategyRuleService;
    @Autowired
    private IRecommendStrategyService strategyService;
    @Autowired
    private RecommendClient recommendClient;

    @GetMapping("/getJson")
    public Response getJson() {
        InputStream ist = RecommendRuleController.class.getClassLoader().getResourceAsStream("rule.json");
        InputStreamReader is = new InputStreamReader(ist);
        JSONReader jsonReader = new JSONReader(is);
        Object o = jsonReader.readObject();
        return Response.ok(o);
    }

    @PostMapping("/template/save")
    @ApiOperation(value = "规则保存(修改)模板, 然后返回内容规则的id")
    public Response<Long> templateSave(
            @ApiParam @Validated @RequestBody RuleVo ruleTemplateSave) {
        Long ruleId;
        if (ruleTemplateSave.getId() != null) {
            ruleId = ruleService.update(ruleTemplateSave, DroolsConstants.TEMPLATE_YES);
        } else {
            ruleId = ruleService.insert(ruleTemplateSave, DroolsConstants.TEMPLATE_YES);
        }
        if (ruleId != null) {
            return Response.ok(ruleId);
        }
        return Response.fail();
    }

    @GetMapping("/page")
    @ApiOperation(value = "规则列表页（引用规则列表页）", response = RulePageVo.class)
    public Response<RulePageVo> page(
            @ApiParam("规则类型，内容规则:1,用户规则:2") @RequestParam Integer type,
            @ApiParam @RequestParam(required = false) String name,
            @RequestParam(defaultValue = "1") Integer pageNo,
            @RequestParam(defaultValue = "10") Integer pageSize) {
        Page<RecommendRule> page = new Page<>(pageNo, pageSize);
        page = ruleService.selectPageByTypeAndName(page, type, name);
        if (page != null) {
            List<RecommendRule> records = page.getRecords();
            if (!CollectionUtils.isEmpty(records)) {
                List<RulePageVo> list = new ArrayList<>();
                records.forEach(item -> {
                    RulePageVo vo = new RulePageVo();
                    vo.setId(item.getId());
                    vo.setCreateTime(item.getCreateTime());
                    vo.setName(item.getName());
                    list.add(vo);
                });
                return ResponseUtil.pageResponse4PageParam(page, list);
            }
        }
        return Response.fail();
    }

    @PostMapping("/ids/get")
    @ApiOperation(value = "根据规则id获取规则详情（回显）", response = RuleItemVo.class)
    public Response<RuleVo> getContentRuleById(@RequestBody ArrayList<String> ruleIds) {
        if (CollectionUtils.isEmpty(ruleIds)) {
            return Response.ok();
        }
        // 组装 RuleVo
        RecommendRule recommendRule = new RecommendRule();
        ContextUtil.buildTenantEntity4Select(recommendRule);
        EntityWrapper<RecommendRule> ew = new EntityWrapper<>(recommendRule);
        ew.in("id", ruleIds);
        List<RecommendRule> recommendRuleList = recommendRule.selectList(ew);
        if (!CollectionUtils.isEmpty(recommendRuleList)) {
            Map<Long, RuleVo> map = new HashMap<>();
            RuleVo ruleVo;
            for (RecommendRule rule : recommendRuleList) {
                ruleVo = new RuleVo();
                ruleVo.setQuoteRuleId(rule.getId());
                ruleVo.setTemplate(true);
                ruleVo.setType(rule.getType());
                ruleVo.setList(new ArrayList<>());
                ruleVo.setName(rule.getName());
                map.put(rule.getId(), ruleVo);
            }
            // 组装 RuleItemVo
            RecommendRuleItem item = new RecommendRuleItem();
            ContextUtil.buildTenantEntity4Select(item);
            EntityWrapper<RecommendRuleItem> itemEw = new EntityWrapper<>(item);
            itemEw.in("rule_id", ruleIds);
            itemEw.orderBy("sort");

            List<RecommendRuleItem> items = item.selectList(itemEw);
            if (!CollectionUtils.isEmpty(items)) {
                RuleItemVo ruleItemVo;
                for (RecommendRuleItem recommendRuleItem : items) {
                    ruleItemVo = new RuleItemVo();
                    ruleItemVo.setSort(recommendRuleItem.getSort());
                    ruleItemVo.setAndOr(recommendRuleItem.getAndOr());
                    ruleItemVo.setBusiness(recommendRuleItem.getBusiness());
                    ruleItemVo.setProperty(recommendRuleItem.getPropertyName());
                    ruleItemVo.setRelation(recommendRuleItem.getRelation());
                    ruleItemVo.setValue(recommendRuleItem.getTargetVal());
                    ruleItemVo.setName(recommendRuleItem.getName());
                    if (map.containsKey(recommendRuleItem.getRuleId())) {
                        map.get(recommendRuleItem.getRuleId()).getList().add(ruleItemVo);
                    }
                }
                return Response.ok(map);
            }
        }
        return Response.fail();
    }

    @ApiOperation("规则复制")
    @PostMapping("/copy")
    public Response copy(@RequestBody HashMap<String, String> map) {
        String str = map.get("id");
        if (StringUtils.isEmpty(str)) {
            return Response.fail("4001", "id不能为空");
        }
        Long id = Long.valueOf(str);
        return Response.ok(ruleService.copy(id));
    }

    @ApiOperation("删除规则")
    @PostMapping("/delete")
    public Response delete(@ApiParam @RequestBody HashMap<String, String> map) {
        String str = map.get("id");
        if (StringUtils.isEmpty(str)) {
            return Response.fail("4001", "id不能为空");
        }
        Long id = Long.valueOf(str);
        // 先查看有没有跟本规则关联的策略
        EntityWrapper ew = new EntityWrapper();
        ew.eq("rule_id", id);
        int count = strategyRuleService.selectCount(ew);
        if (count > 0) {
            return Response.fail("4001", "当前规则已被策略引用");
        }
        boolean flag = ruleService.deleteById(id);
        if (flag) {
            ew = new EntityWrapper();
            ew.eq("rule_id", id);
            boolean resFlag = ruleItemService.delete(ew);
        }
        return Response.ok();
    }

    @ApiOperation("查看关联活动(关联策略)")
    @GetMapping("/strategy/page")
    public Response<StrategyPageVo> strategyPage(
            @ApiParam @RequestParam("ruleId") Long ruleId,
            @ApiParam @RequestParam(required = false) String name,
            @RequestParam(defaultValue = "1") Integer pageNo,
            @RequestParam(defaultValue = "10") Integer pageSize
    ) {
        Page<StrategyPageVo> page = strategyService.pageByNameAndRuleId(name, ruleId, new Page(pageNo, pageSize));
        if (page != null) {
            return ResponseUtil.pageResponse4PageParam(page, page.getRecords());
        }
        return Response.fail();
    }

    @ApiOperation(value = "查看规则范围", response = RuleContentRangeVo.class)
    @GetMapping("/content/range/page")
    public Response<RuleContentRangeVo> conteRangePage(
            @ApiParam("id") @RequestParam("id") Long id,
            @ApiParam("关键字") @RequestParam(name = "keyword", required = false) String keyword,
            @ApiParam("内容类型") @RequestParam(name = "business", required = false) String business,
            @RequestParam(defaultValue = "1", required = false) Integer pageNo,
            @RequestParam(defaultValue = "10", required = false) Integer pageSize
    ) {
        RecommendRule rule = ruleService.selectById(id);
        if (rule == null) {
            return Response.fail("规则不存在");
        }

        EntityWrapper ew = new EntityWrapper();
        ew.eq("rule_id", id);
        if (StringUtils.isNotEmpty(business)) {
            ew.eq("business", business);
        }
        ew.orderBy("sort");

        List<RecommendRuleItem> ruleList = ruleItemService.selectList(ew);
        if (!CollectionUtils.isEmpty(ruleList)) {
            List<RuleItemVo> ruleItemVos = new ArrayList<>(ruleList.size());
            for (RecommendRuleItem ruleItem : ruleList) {
                RuleItemVo itemVo = new RuleItemVo();
                itemVo.setBusiness(ruleItem.getBusiness());
                itemVo.setAndOr(ruleItem.getAndOr());
                itemVo.setProperty(ruleItem.getPropertyName());
                itemVo.setRelation(ruleItem.getRelation());
                itemVo.setValue(ruleItem.getTargetVal());
                itemVo.setSort(ruleItem.getSort());
                itemVo.setName(ruleItem.getName());
                ruleItemVos.add(itemVo);
            }
            BusinessVo4PortalParam param = new BusinessVo4PortalParam();
            List<RuleVo> ruleVos = new ArrayList<>();

            RuleVo ruleVo = new RuleVo();
            ruleVo.setType(rule.getType());
            ruleVo.setAndOr(DroolsConstants.AND);
            ruleVo.setList(ruleItemVos);
            ruleVos.add(ruleVo);

            param.setRuleVos(ruleVos);
            param.setKeyword(keyword);
            param.setPageNo(pageNo);
            param.setPageSize(pageSize);
            param.setRequestContext(ContextHolder.get());
            BusinessVo4PortalResponse response = recommendClient.recommend4Manage(param);

            log.info("-------------------------------------------" + response.toString());

            Map<String, Integer> pageMap = new HashMap<>();
            pageMap.put("pageNo", response.getPageNo());
            pageMap.put("pageSize", response.getPageSize());
            pageMap.put("pageTotal", response.getTotal());

            List<BusinessVo4Portal> vo4Portals = response.getList();
            if (!CollectionUtils.isEmpty(vo4Portals)) {
                List<RuleContentRangeVo> vos = new ArrayList<>(vo4Portals.size());
                for (int i = 0; i < vo4Portals.size(); i++) {
                    BusinessVo4Portal p = vo4Portals.get(i);
                    RuleContentRangeVo vo = new RuleContentRangeVo();
                    vo.setNum(i + 1);
                    vo.setId(vo.getId());
                    vo.setContent(p.getName());
                    vo.setCode(p.getCode());
                    vo.setBusiness(DroolsConstants.getBusinessCHN(p.getType()));
                    vos.add(vo);
                }
                return Response.ok(vos, pageMap);
            }
        }
        return Response.ok();
    }


    @ApiOperation("查看用户规则范围")
    @GetMapping("/getVisibleRangeToUser")
    public Response getVisibleRange(@ApiParam("规则id") @RequestParam("id") Long id,
                                    @ApiParam("查询keyWord") @RequestParam(value = "keyword", required = false) String keyword,
                                    @RequestParam(defaultValue = "1", required = false) Integer pageNo,
                                    @RequestParam(defaultValue = "10", required = false) Integer pageSize) {
        RecommendRule rule = ruleService.selectById(id);
        if (rule == null) {
            return Response.fail("规则不存在");
        }

        EntityWrapper ew = new EntityWrapper();
        ew.eq("rule_id", id);
        ew.orderBy("sort");

        List<RecommendRuleItem> ruleList = ruleItemService.selectList(ew);
        if (!CollectionUtils.isEmpty(ruleList)) {
            List<RuleItemVo> ruleItemVos = new ArrayList<>(ruleList.size());
            for (RecommendRuleItem ruleItem : ruleList) {
                RuleItemVo itemVo = new RuleItemVo();
                itemVo.setBusiness(ruleItem.getBusiness());
                itemVo.setAndOr(ruleItem.getAndOr());
                itemVo.setProperty(ruleItem.getPropertyName());
                itemVo.setRelation(ruleItem.getRelation());
                itemVo.setValue(ruleItem.getTargetVal());
                itemVo.setSort(ruleItem.getSort());
                itemVo.setName(ruleItem.getName());
                ruleItemVos.add(itemVo);
            }
            AccountVo4ManageParam param = new AccountVo4ManageParam();
            List<RuleVo> ruleVos = new ArrayList<>();

            RuleVo ruleVo = new RuleVo();
            ruleVo.setType(rule.getType());
            ruleVo.setAndOr(DroolsConstants.AND);
            ruleVo.setList(ruleItemVos);
            ruleVos.add(ruleVo);

            param.setRuleVos(ruleVos);
            param.setKeyword(keyword);
            param.setPageNo(pageNo);
            param.setPageSize(pageSize);
            param.setRequestContext(ContextHolder.get());
            AccountVo4ManageResponse response = recommendClient.recommend4ManageAccount(param);
            if (response == null) {
                return Response.ok();
            }

            log.info("-------------------------------------------" + response);

            Map<String, Integer> pageMap = new HashMap<>();
            pageMap.put("pageNo", response.getPageNo());
            pageMap.put("pageSize", response.getPageSize());
            pageMap.put("pageTotal", response.getTotal());

            List<AccountVo4Manage> vo4Manages = response.getList();
            return Response.ok(vo4Manages, pageMap);
        }
        return Response.ok();
    }

    // ***********************************  ********************************************


}