package com.yizhi.drools.application.controller.manage;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.drools.application.constant.DroolsConstants;
import com.yizhi.drools.application.domain.RecommendStrategy;
import com.yizhi.drools.application.feign.RecommendClient;
import com.yizhi.drools.application.service.IRecommendRuleItemService;
import com.yizhi.drools.application.service.IRecommendRuleService;
import com.yizhi.drools.application.service.IRecommendStrategyRuleService;
import com.yizhi.drools.application.service.IRecommendStrategyService;
import com.yizhi.drools.application.util.ContextUtil;
import com.yizhi.drools.application.util.ResponseUtil;
import com.yizhi.drools.application.vo.drools.*;
import com.yizhi.drools.application.vo.manage.RuleContentRangeVo;
import com.yizhi.drools.application.vo.manage.RuleVo;
import com.yizhi.drools.application.vo.manage.StrategyPageVo;
import com.yizhi.drools.application.vo.manage.StrategyVo;
import com.yizhi.util.application.domain.Response;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author shengchenglong123
 * @since 2019-11-27
 */
@Api(tags = "推荐策略 - 管理端接口")
@RestController
@RequestMapping("/manage/recommendStrategy")
@Slf4j
public class RecommendStrategyController {

    @Autowired
    private IRecommendStrategyService strategyService;
    @Autowired
    private IRecommendRuleService ruleService;
    @Autowired
    private IRecommendRuleItemService ruleItemService;
    @Autowired
    private IRecommendStrategyRuleService strategyRuleService;
    @Autowired
    private RecommendClient recommendClient;

    @PostMapping("/save")
    @ApiOperation(value = "内容规则保存模板, 然后返回内容规则的id")
    public Response save(@ApiParam @Validated @RequestBody StrategyVo strategyVo) {
        try {

            RecommendStrategy strategy = new RecommendStrategy();
            strategy.setName(strategyVo.getStrategyName());
            ContextUtil.buildTenantEntity4Select(strategy);

            EntityWrapper ew = new EntityWrapper(strategy);
            if (strategyVo.getId() != null) {
                ew.ne("id", strategyVo.getId());
            }
            int count = strategyService.selectCount(ew);
            if (count > 0) {
                return Response.fail("4001", "策略名称已经存在！");
            }

            boolean flag = strategyService.save(strategyVo);
            if (flag) {
                return Response.ok(flag);
            }
            return Response.fail();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.fail("4001", e.getLocalizedMessage());
        }
    }

    @GetMapping("/page")
    @ApiOperation(value = "策略列表分页", response = StrategyPageVo.class)
    public Response<StrategyPageVo> listPage(
            @ApiParam("策略名称") @RequestParam(required = false) String name,
            @ApiParam("开始时间") @RequestParam(required = false) String startTime,
            @ApiParam("结束时间") @RequestParam(required = false) String endTime,
            @ApiParam("是否是门户配置的列表") @RequestParam(required = false) Boolean isForPortal,
            @RequestParam(name = "", defaultValue = "1") Integer pageNo,
            @RequestParam(defaultValue = "10") Integer pageSize
    ) {
        Page<RecommendStrategy> page = new Page<>(pageNo, pageSize);
        RecommendStrategy strategy = new RecommendStrategy();
        ContextUtil.buildTenantEntity4Select(strategy);
        EntityWrapper<RecommendStrategy> ew = new EntityWrapper<>(strategy);
        if (StringUtils.isNotEmpty(name)) {
            ew.like("name", name);
        }

        Date startTimeDate = null;
        Date endTimeDate = null;

        if (StringUtils.isNotEmpty(startTime)) {
            startTimeDate = DateUtil.beginOfDay(new Date(Long.valueOf(startTime)));
        }
        if (StringUtils.isNotEmpty(endTime)) {
            endTimeDate = DateUtil.endOfDay(new Date(Long.valueOf(endTime)));
        }

        //通过时间查询时  带上长期有效的结果
        if (startTime != null && endTime != null) {
            ew.andNew().gt("start_time", startTimeDate).and().lt("end_time", endTimeDate).or().eq("time_type",1);
        } else {
            if (startTime != null) {
                ew.gt("start_time", startTimeDate).or().eq("time_type",1);
            }
            if (endTime != null) {
                ew.lt("end_time", endTimeDate).or().eq("time_type",1);
            }
        }

        ew.orderBy("create_time", false);
        ew.orderBy("update_time", false);
        page = strategyService.selectPage(page, ew);
        Date date=new Date();
        List<RecommendStrategy> strategies = page.getRecords();
        List<StrategyPageVo> list = new ArrayList<>();
        if (!CollectionUtils.isEmpty(strategies)) {
            strategies.forEach(item -> {

                StrategyPageVo vo = new StrategyPageVo();
                vo.setCreateTime(item.getCreateTime());
                vo.setEndTime(item.getEndTime());
                vo.setName(item.getName());
                vo.setStartTime(item.getStartTime());
                vo.setId(item.getId());
                list.add(vo);
            });
        }
        return ResponseUtil.pageResponse4PageParam(page, list);
    }

    @ApiOperation("查看策略")
    @GetMapping("/view")
    public Response view(@ApiParam @RequestParam(name = "id") Long id) {
        RecommendStrategy strategy = strategyService.selectById(id);
        if (strategy == null) {
            return Response.fail("4001", "该id不存在");
        }
        // 组装 策略vo
        StrategyVo strategyVo = new StrategyVo();
        strategyVo.setTimeType(strategy.getTimeType());
        strategyVo.setEndTime(strategy.getEndTime());
        strategyVo.setStartTime(strategy.getStartTime());
        strategyVo.setStrategyName(strategy.getName());
        strategyVo.setRuleVos(ruleService.listRuleVosByStrategyId(id, false, null));
        return Response.ok(strategyVo);
    }

    @PostMapping("/copy")
    @ApiOperation(value = "策略复制")
    public Response copy(@ApiParam("需要复制的策略id") @RequestBody HashMap<String, String> map) {
        String str = map.get("id");
        if (StringUtils.isEmpty(str)) {
            return Response.fail("4001", "id不能为空");
        }
        Long id = Long.valueOf(str);
        try {
            strategyService.copy(id);
            return Response.ok();
        } catch (Exception e) {
            e.printStackTrace();
            return Response.fail("4001", e.getLocalizedMessage());
        }
    }

    @ApiOperation("删除策略")
    @PostMapping("/delete")
    public Response delete(@ApiParam @RequestBody HashMap<String, String> map) {
        // TODO 先查看本策略有没有被使用
        String str = map.get("id");
        if (StringUtils.isEmpty(str)) {
            return Response.fail("4001", "id不能为空");
        }
        Long id = Long.valueOf(str);
        boolean flag = strategyService.delete(id);
        return Response.ok();
    }

    @ApiOperation(value = "查看规则范围", response = RuleContentRangeVo.class)
    @GetMapping("/content/range/page")
    public Response<RuleContentRangeVo> contentRangePage(
            @ApiParam("id") @RequestParam("id") Long id,
            @ApiParam("关键字") @RequestParam(name = "keyword", required = false) String keyword,
            @ApiParam("内容类型") @RequestParam(name = "business", required = false) String business,
            @RequestParam(defaultValue = "1", required = false) Integer pageNo,
            @RequestParam(defaultValue = "10", required = false) Integer pageSize
    ) {
        List<RuleVo> ruleList = ruleService.listRuleVosByStrategyId(id, true, business);
        if (!CollectionUtils.isEmpty(ruleList)) {

            BusinessVo4PortalParam param = new BusinessVo4PortalParam();
            param.setRuleVos(ruleList);
            if (StringUtils.isNotEmpty(keyword)) {
                param.setKeyword(keyword);
            }
            if (StringUtils.isNotEmpty(business)) {
                param.setBusiness(business);
            }
            param.setPageNo(pageNo);
            param.setPageSize(pageSize);
            param.setRequestContext(ContextHolder.get());
            BusinessVo4PortalResponse response = recommendClient.recommend4Manage(param);

            log.info("-------------------------------------------" + response.toString());

            Map<String, Integer> pageMap = new HashMap<>();
            pageMap.put("pageNo", response.getPageNo());
            pageMap.put("pageSize", response.getPageSize());
            pageMap.put("pageTotal", response.getTotal());

            List<BusinessVo4Portal> vo4Portals = response.getList();
            if (!CollectionUtils.isEmpty(vo4Portals)) {
                List<RuleContentRangeVo> vos = new ArrayList<>(vo4Portals.size());
                for (int i = 0; i < vo4Portals.size(); i++) {
                    BusinessVo4Portal p = vo4Portals.get(i);
                    RuleContentRangeVo vo = new RuleContentRangeVo();
                    vo.setNum(i + 1);
                    vo.setId(vo.getId());
                    vo.setContent(p.getName());
                    vo.setCode(p.getCode());
                    vo.setBusiness(DroolsConstants.getBusinessCHN(p.getType()));
                    vos.add(vo);
                }
                return Response.ok(vos, pageMap);
            }
        }
        return Response.ok();
    }

    @ApiOperation("查看用户规则范围")
    @GetMapping("/getVisibleRangeToUser")
    public Response getVisibleRange(@ApiParam("策略id") @RequestParam("id") Long id,
                                    @ApiParam("查询keyWord") @RequestParam(value = "keyword", required = false) String keyword,
                                    @RequestParam(defaultValue = "1", required = false) Integer pageNo,
                                    @RequestParam(defaultValue = "10", required = false) Integer pageSize) {

        RecommendStrategy recommendStrategy = strategyService.selectById(id);
        if (recommendStrategy == null) {
            return Response.fail("该策略不存在！");
        }
        List<RuleVo> ruleList = ruleService.listRuleVosByStrategyId(id, true, null);
        if (!CollectionUtils.isEmpty(ruleList)) {
            AccountVo4ManageParam param = new AccountVo4ManageParam();
            param.setRuleVos(ruleList);
            if (StringUtils.isNotEmpty(keyword)) {
                param.setKeyword(keyword);
            }

            param.setPageNo(pageNo);
            param.setPageSize(pageSize);
            param.setRequestContext(ContextHolder.get());
            AccountVo4ManageResponse response = recommendClient.recommend4ManageAccount(param);

            log.info("-------------------------------------------" + response.toString());

            Map<String, Integer> pageMap = new HashMap<>();
            pageMap.put("pageNo", response.getPageNo());
            pageMap.put("pageSize", response.getPageSize());
            pageMap.put("pageTotal", response.getTotal());

            List<AccountVo4Manage> vo4Manages = response.getList();
            return Response.ok(vo4Manages, pageMap);
        }
        return Response.ok();
    }

    // ********************************* remote api **********************************

    /**
     * 门户配置的策略回显
     *
     * @param strategyIds
     * @return
     */
    @PostMapping("/list4PortalConfig")
    public Map<Long, StrategyPageVo> list4PortalConfig(@RequestBody List<Long> strategyIds) {
        if (!CollectionUtils.isEmpty(strategyIds)) {
            EntityWrapper<RecommendStrategy> recommendStrategyEntityWrapper = new EntityWrapper<>();
            recommendStrategyEntityWrapper.in("id", strategyIds);
            List<RecommendStrategy> strategyList = strategyService.selectList(recommendStrategyEntityWrapper);
            if (!CollectionUtils.isEmpty(strategyList)) {
                Map<Long, StrategyPageVo> map = new HashMap<>(strategyList.size());
                strategyList.forEach(strategy -> {
                    StrategyPageVo vo = new StrategyPageVo();
                    vo.setId(strategy.getId());
                    vo.setCreateTime(strategy.getCreateTime());
                    vo.setEndTime(strategy.getEndTime());
                    vo.setName(strategy.getName());
                    vo.setStartTime(strategy.getStartTime());

                    map.put(vo.getId(), vo);
                });
                return map;
            }
        }
        return null;
    }

    // ************************************ 尝试组装一个 *****************************************

    /**
     * 返回给管理端的推荐内容
     *
     * @param param
     * @return
     */
    @PostMapping("/businessVo4Manage")
    public BusinessVo4PortalResponse businessVo4Manage(@RequestBody BusinessVo4PortalParam param) {
        List<Long> strategyIds = param.getStrategyIds();
        if (!CollectionUtils.isEmpty(strategyIds)) {
            //去除已经过期的策略id
            deletePastDueStrategy(strategyIds);
            List<RuleVo> vos = new ArrayList<>();
            for (Long strategyId : strategyIds) {
                vos.addAll(ruleService.listRuleVosByStrategyId(strategyId, true, null));
            }
            param.setRuleVos(vos);
            param.setRequestContext(ContextHolder.get());
            BusinessVo4PortalResponse businessVo4Portals = recommendClient.recommend4Manage(param);
            log.info("--------------------- " + businessVo4Portals);
            return businessVo4Portals;
        }
        return null;
    }

    /**
     * 返回给学员端的推荐内容
     *
     * @param param
     * @return
     */
    @PostMapping("/businessVo4Portal")
    public BusinessVo4PortalResponse businessVo4Portal(@RequestBody BusinessVo4PortalParam param) {
        // 先查询这个用户在不在推荐范围内
        List<Long> strategyIds = param.getStrategyIds();
        if (!CollectionUtils.isEmpty(strategyIds)) {
            //去除已经过期的策略id
            deletePastDueStrategy(strategyIds);
            List<RuleVo> vos = new ArrayList<>();
            for (Long strategyId : strategyIds) {
                param.getStrategyRuleVoMap().put(strategyId, ruleService.listRuleVosByStrategyId(strategyId, true, null));
            }
            param.setRequestContext(ContextHolder.get());
            BusinessVo4PortalResponse businessVo4Portals = recommendClient.recommend4Portal(param);
            log.info("--------------------- " + businessVo4Portals);
            return businessVo4Portals;
        }
        return null;
    }

    private void deletePastDueStrategy(List<Long> strategyIds) {
        Date date=new Date();
        EntityWrapper<RecommendStrategy> wrapper = new EntityWrapper<>();
        wrapper.in("id",strategyIds);
        List<RecommendStrategy> recommendStrategies=strategyService.selectList(wrapper);
        if (recommendStrategies.isEmpty()){
            return;
        }
        for (RecommendStrategy r:recommendStrategies) {
            if (r.getTimeType()==2&&r.getEndTime().getTime()<date.getTime()){
                strategyIds.remove(r.getId());
            }
        }
    }
}