package com.yizhi.drools.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.toolkit.StringUtils;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.drools.application.constant.DroolsConstants;
import com.yizhi.drools.application.domain.RecommendRule;
import com.yizhi.drools.application.domain.RecommendRuleItem;
import com.yizhi.drools.application.domain.RecommendStrategyRule;
import com.yizhi.drools.application.mapper.RecommendRuleItemMapper;
import com.yizhi.drools.application.mapper.RecommendRuleMapper;
import com.yizhi.drools.application.service.IRecommendRuleItemService;
import com.yizhi.drools.application.service.IRecommendRuleService;
import com.yizhi.drools.application.service.IRecommendStrategyRuleService;
import com.yizhi.drools.application.util.ContextUtil;
import com.yizhi.drools.application.vo.manage.RuleItemVo;
import com.yizhi.drools.application.vo.manage.RuleVo;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.DroolsAccountVO;
import com.yizhi.system.application.vo.RuleVisibleRangeVo;
import com.yizhi.system.application.vo.SystemRuleItemVo;
import com.yizhi.util.application.page.PageInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.yizhi.application.orm.id.IdGenerator;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author shengchenglong123
 * @since 2019-11-27
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class RecommendRuleServiceImpl extends ServiceImpl<RecommendRuleMapper, RecommendRule> implements IRecommendRuleService {

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private IRecommendRuleItemService ruleItemService;
    @Autowired
    private RecommendRuleItemMapper ruleItemMapper;
    @Autowired
    private IRecommendStrategyRuleService strategyRuleService;
    @Autowired
    private AccountClient accountClient;


    @Override
    public Long insert(RuleVo ruleVo, Integer isTemplate) {
        RecommendRule rule = new RecommendRule();
        rule.setName(ruleVo.getName());
        rule.setId(idGenerator.generate());
        rule.setDeleted(DroolsConstants.DELETED_NO);
        rule.setTemplate(isTemplate);
        rule.setType(ruleVo.getType());
        ContextUtil.buildTenantEntity4Insert(rule);

        if (rule.insert()) {
            ruleSaveThenDealRuleItem(ruleVo, rule.getId());
        }
        return rule.getId();
    }

    @Override
    public Long update(RuleVo ruleVo, Integer isTemplate) {
        RecommendRule rule = new RecommendRule();
        rule.setId(ruleVo.getId());
        rule.setName(ruleVo.getName());
        rule.setDeleted(DroolsConstants.DELETED_NO);
        rule.setTemplate(isTemplate);
        rule.setType(ruleVo.getType());

        // 修改rule
        int num = this.baseMapper.updateById(rule);
        if (num == 1) {
            // 删除以前的 ruleItem
            EntityWrapper ew = new EntityWrapper();
            ew.eq("rule_id", rule.getId());
            int ruleItemDelNum = ruleItemMapper.delete(ew);
            ruleSaveThenDealRuleItem(ruleVo, rule.getId());
        }
        return rule.getId();
    }

    @Override
    public Page<RecommendRule> selectPageByTypeAndName(Page<RecommendRule> page, Integer type, String name) {
        RecommendRule rule = new RecommendRule();
        ContextUtil.buildTenantEntity4Select(rule);
        rule.setType(type);
        rule.setTemplate(DroolsConstants.TEMPLATE_YES);

        EntityWrapper<RecommendRule> ew = new EntityWrapper<>(rule);
        if (StringUtils.isNotEmpty(name)) {
            ew.like("name", name);
        }
        ew.orderBy("create_time", false);
        ew.orderBy("update_time", false);
        page = rule.selectPage(page, ew);
        return page;
    }

    @Override
    public List<RuleVo> listRuleVosByStrategyId(Long strategyId, boolean withClassName, String business) {
        List<RecommendStrategyRule> strategyRuleList = strategyRuleService.listByStrategyId(strategyId);
        List<RuleVo> ruleVos = new ArrayList<>();
        if (!org.springframework.util.CollectionUtils.isEmpty(strategyRuleList)) {
            strategyRuleList.forEach(strategyRule -> {
                RecommendRule rule = this.baseMapper.selectById(strategyRule.getRuleId());
                if (rule != null) {
                    // 组装 ruleVo
                    RuleVo ruleVo = new RuleVo();
                    ruleVo.setAndOr(strategyRule.getAndOr());
                    ruleVo.setType(rule.getType());
                    ruleVo.setTemplate(DroolsConstants.TEMPLATE_YES.equals(rule.getTemplate()));
                    ruleVo.setName(rule.getName());
                    ruleVo.setSort(strategyRule.getSort());
                    if (rule.getTemplate().equals(DroolsConstants.RULE_TYPE_CONTENT)) {
                        ruleVo.setQuoteRuleId(rule.getId());
                    }

                    // 组装 ruleItemVo
                    List<RecommendRuleItem> ruleItemList = ruleItemService.listByRuleId(rule.getId(), business);
                    if (!org.springframework.util.CollectionUtils.isEmpty(ruleItemList)) {
                        List<RuleItemVo> list = new ArrayList<>(ruleItemList.size());
                        ruleItemList.forEach(ruleItem -> {
                            RuleItemVo vo = new RuleItemVo();
                            vo.setSort(ruleItem.getSort());
                            vo.setAndOr(ruleItem.getAndOr());
                            vo.setBusiness(ruleItem.getBusiness());
                            vo.setProperty(ruleItem.getPropertyName());
                            vo.setRelation(ruleItem.getRelation());
                            vo.setValue(ruleItem.getTargetVal());
                            vo.setName(ruleItem.getName());
                            if (withClassName) {
                                vo.setBusinessClassName(ruleItem.getClassName());
                            }
                            list.add(vo);
                        });
                        ruleVo.setList(list);
                    }
                    ruleVos.add(ruleVo);
                }
            });
        }
        return ruleVos;
    }

    @Override
    public boolean copy(Long ruleId) {

        String currentTimeStr = "-复制-".concat(String.valueOf(System.currentTimeMillis()));
        RecommendRule ruleTarget;
        List<RecommendRule> ruleListTarget = new ArrayList<>();
        List<RecommendRuleItem> ruleItemListTarget = new ArrayList<>();
        RecommendRule rule = this.baseMapper.selectById(ruleId);

        if (rule != null) {
            // 复制 rule
            ruleTarget = copyRule(currentTimeStr, rule);
            ruleListTarget.add(ruleTarget);
            // 复制 rule_item
            ruleItemListTarget.addAll(copyRuleItem(rule.getId(), ruleTarget.getId()));
        }
        if (!CollectionUtils.isEmpty(ruleListTarget)) {
            this.insertBatch(ruleListTarget);
        }
        if (!CollectionUtils.isEmpty(ruleItemListTarget)) {
            ruleItemService.insertBatch(ruleItemListTarget);
        }
        return true;
    }

    /**
     * 修改，新增 rule 时的 ruleItem保存
     *
     * @param ruleVo
     * @param ruleId
     * @return
     */
    private boolean ruleSaveThenDealRuleItem(RuleVo ruleVo, long ruleId) {
        List<RuleItemVo> list = ruleVo.getList();
        if (CollectionUtils.isEmpty(list)) {
            return true;
        }
        List<RecommendRuleItem> ruleItems = new ArrayList<>(list.size());
        // 组装每一个ruleItem
        for (RuleItemVo templateSave : list) {
            RecommendRuleItem ruleItem = new RecommendRuleItem();
            ruleItem.setRuleId(ruleId);
            ruleItem.setBusiness(templateSave.getBusiness());
            ruleItem.setRelation(templateSave.getRelation());
            ruleItem.setDeleted(DroolsConstants.DELETED_NO);
            ruleItem.setId(idGenerator.generate());
            ruleItem.setAndOr(templateSave.getSort().equals(1) ? DroolsConstants.AND : templateSave.getAndOr());
            ruleItem.setPropertyName(templateSave.getProperty());
            ruleItem.setSort(templateSave.getSort());
            ruleItem.setTargetVal(templateSave.getValue());
            ruleItem.setType(ruleVo.getType());
            ruleItem.setName(templateSave.getName());
            if (ruleItem.getType().equals(DroolsConstants.RULE_TYPE_CONTENT)) {
                ruleItem.setClassName(templateSave.getBusiness());
            } else {
                ruleItem.setClassName(DroolsConstants.BUSINESS_CLASS_NAME_USER);
                ruleItem.setBusiness("account");
            }
            ContextUtil.buildTenantEntity4Insert(ruleItem);
            ruleItems.add(ruleItem);
        }
        if (!CollectionUtils.isEmpty(ruleItems)) {
            return ruleItemService.insertBatch(ruleItems);
        }
        return false;
    }

    private RecommendRule copyRule(String currentTimeStr, RecommendRule origin) {
        RecommendRule ruleTarget = new RecommendRule();
        ruleTarget.setType(origin.getType());
        ruleTarget.setDeleted(DroolsConstants.DELETED_NO);
        ruleTarget.setId(idGenerator.generate());
        ruleTarget.setTemplate(origin.getTemplate());
        ruleTarget.setName(origin.getName().concat(currentTimeStr));
        ContextUtil.buildTenantEntity4Insert(ruleTarget);
        return ruleTarget;
    }

    private List<RecommendRuleItem> copyRuleItem(Long ruleId, Long ruleIdTarget) {
        RecommendRuleItem ruleItem = new RecommendRuleItem();
        ruleItem.setRuleId(ruleId);
        ContextUtil.buildTenantEntity4Select(ruleItem);
        List<RecommendRuleItem> ruleItemList = ruleItem.selectList(new EntityWrapper(ruleItem));
        List<RecommendRuleItem> ruleItemListTarget = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(ruleItemList)) {
            RecommendRuleItem ruleItemTarget = null;
            for (RecommendRuleItem item : ruleItemList) {
                ruleItemTarget = new RecommendRuleItem();
                ruleItemTarget.setId(idGenerator.generate());
                ruleItemTarget.setName(item.getName());
                ruleItemTarget.setBusiness(item.getBusiness());
                ruleItemTarget.setRuleId(ruleIdTarget);
                ruleItemTarget.setAndOr(item.getAndOr());
                ruleItemTarget.setClassName(item.getClassName());
                ruleItemTarget.setDeleted(DroolsConstants.DELETED_NO);
                ruleItemTarget.setPropertyName(item.getPropertyName());
                ruleItemTarget.setRelation(item.getRelation());
                ruleItemTarget.setSort(item.getSort());
                ruleItemTarget.setTargetVal(item.getTargetVal());
                ruleItemTarget.setType(item.getType());
                ContextUtil.buildTenantEntity4Insert(ruleItemTarget);
                ruleItemListTarget.add(ruleItemTarget);
            }
        }
        return ruleItemListTarget;
    }

    @Override
    public Page getVisibleRange(Long id, String keyword, Integer pageNo, Integer pageSize) {

        RequestContext context = ContextHolder.get();
        Page page = new Page(pageNo, pageSize);
        List<RecommendRuleItem> items = ruleItemService.listByRuleId(id, null);
        if (CollectionUtils.isNotEmpty(items)) {
            RuleVisibleRangeVo rangeVo = new RuleVisibleRangeVo();
            rangeVo.setKeyword(keyword);
            rangeVo.setPageNo(pageNo);
            rangeVo.setPageSize(pageSize);
            rangeVo.setCompanyId(context.getCompanyId());
            rangeVo.setSiteId(context.getSiteId());
            List<SystemRuleItemVo> ruleItemVos = new ArrayList<>(items.size());
            items.forEach(a -> {
                //只获取用户规则的项
                if (DroolsConstants.RULE_TYPE_USER.equals(a.getType())) {
                    SystemRuleItemVo vo = new SystemRuleItemVo();
                    BeanUtils.copyProperties(a, vo);
                    vo.setId(a.getId());
                    ruleItemVos.add(vo);
                }
            });
            rangeVo.setRuleItemVos(ruleItemVos);
            PageInfo<DroolsAccountVO> pageByDrools = accountClient.getPageByRuleItems(rangeVo);
            page.setRecords(pageByDrools.getRecords());
            page.setTotal(pageByDrools.getPageRecords());
        }
        return page;
    }

//    @Override
//    public Map<String, QueryBuilder> builderQueryBuilder(Long id) {
//        RecommendRuleItemVo ruleItem = new RecommendRuleItemVo();
//        ContextUtil.buildTenantEntity4Select(ruleItem);
//        EntityWrapper ew = new EntityWrapper(ruleItem);
//        ew.eq("rule_id", id);
//        ew.orderBy("sort");
//
//        List<RecommendRuleItemVo> ruleItemList = this.baseMapper.selectList(ew);
//        if (CollectionUtils.isNotEmpty(ruleItemList)) {
//            QueryBuilder queryBuilder = QueryBuilders.boolQuery();
//            ruleItemList.forEach(item -> {
//                item.
//
//
//            });
//        }
//
//
//
//
//        return null;
//    }
}
