package com.yizhi.drools.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.drools.application.constant.DroolsConstants;
import com.yizhi.drools.application.domain.RecommendRule;
import com.yizhi.drools.application.domain.RecommendRuleItem;
import com.yizhi.drools.application.domain.RecommendStrategy;
import com.yizhi.drools.application.domain.RecommendStrategyRule;
import com.yizhi.drools.application.mapper.RecommendRuleMapper;
import com.yizhi.drools.application.mapper.RecommendStrategyMapper;
import com.yizhi.drools.application.service.IRecommendRuleItemService;
import com.yizhi.drools.application.service.IRecommendRuleService;
import com.yizhi.drools.application.service.IRecommendStrategyRuleService;
import com.yizhi.drools.application.service.IRecommendStrategyService;
import com.yizhi.drools.application.util.ContextUtil;
import com.yizhi.drools.application.vo.manage.RuleVo;
import com.yizhi.drools.application.vo.manage.StrategyPageVo;
import com.yizhi.drools.application.vo.manage.StrategyVo;
import com.yizhi.system.application.system.remote.AccountClient;
import com.yizhi.system.application.vo.DroolsAccountVO;
import com.yizhi.system.application.vo.RuleVisibleRangeVo;
import com.yizhi.system.application.vo.SystemRuleItemVo;
import com.yizhi.util.application.page.PageInfo;
import org.apache.ibatis.session.RowBounds;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author shengchenglong123
 * @since 2019-11-27
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class RecommendStrategyServiceImpl extends ServiceImpl<RecommendStrategyMapper, RecommendStrategy> implements IRecommendStrategyService {

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private IRecommendRuleService ruleService;
    @Autowired
    private RecommendRuleMapper ruleMapper;
    @Autowired
    private IRecommendStrategyRuleService strategyRuleService;
    @Autowired
    private IRecommendRuleItemService ruleItemService;
    @Autowired
    private RecommendStrategyMapper strategyMapper;
    @Autowired
    private AccountClient accountClient;

    @Override
    public boolean save(StrategyVo strategyVo) throws Exception {

        RecommendStrategy strategy = new RecommendStrategy();
        ContextUtil.buildTenantEntity4Insert(strategy);
        strategy.setDeleted(DroolsConstants.DELETED_NO);

        strategy.setName(strategyVo.getStrategyName());
        strategy.setTimeType(strategyVo.getTimeType());
        // 自定义时间
        if (strategy.getTimeType().equals(DroolsConstants.STRATEGY_TIME_CUSTOM)) {
            strategy.setStartTime(strategyVo.getStartTime());
            strategy.setEndTime(strategyVo.getEndTime());
        }

        Long oldId = strategyVo.getId();
        boolean flag;
        // 修改
        if (oldId != null) {
            strategy.setId(oldId);
            flag = strategyMapper.updateById(strategy).equals(1);
            if (flag) {
                // 查出之前关联的 rule
                List<Long> ruleIds = strategyRuleService.getRuleIds(oldId);
                if (!CollectionUtils.isEmpty(ruleIds)) {
                    // 删除 strategy-rule 关联
                    EntityWrapper ew = new EntityWrapper();
                    ew.eq("strategy_id", oldId);
                    flag = flag && strategyRuleService.delete(ew);
                    // 删除 非模板的 rule
                    ew = new EntityWrapper();
                    ew.in("id", ruleIds);
                    ew.eq("template", DroolsConstants.TEMPLATE_NO);
                    flag = flag && ruleService.delete(ew);
                    // 删除 非模板的 ruleItem
                    ruleIds = ruleMapper.listTemplateYesOrNoRuleIds(ruleIds, DroolsConstants.TEMPLATE_NO);
                    if (!CollectionUtils.isEmpty(ruleIds)) {
                        ew = new EntityWrapper();
                        ew.in("rule_id", ruleIds);
                        flag = flag && ruleItemService.delete(ew);
                    }
                }
            }
        }
        // 新增
        else {
            strategy.setId(idGenerator.generate());
            flag = strategyMapper.insert(strategy).equals(1);
        }

        // 插入 rule
        if (flag) {
            // 插入规则
            List<RuleVo> ruleVos = strategyVo.getRuleVos();
            List<RecommendStrategyRule> strategyRules = new ArrayList<>();

            Long ruleId;
            RecommendStrategyRule strategyRule;

            for (RuleVo ruleVo : ruleVos) {
                // 如果该规则是引用的
                if (ruleVo.getQuoteRuleId() != null) {
                    ruleId = ruleVo.getQuoteRuleId();
                }
                // 否则是新增的规则
                else {
                    ruleId = ruleService.insert(ruleVo, DroolsConstants.TEMPLATE_NO);
                    if (ruleId == null) {
                        throw new Exception("---------------- 插入规则时，失败 ----------------");
                    }
                }
                // 组装 策略与规则的关联关系
                strategyRule = new RecommendStrategyRule();
                ContextUtil.buildTenantEntity4Insert(strategyRule);
                strategyRule.setStrategyId(strategy.getId());
                strategyRule.setId(idGenerator.generate());
                strategyRule.setAndOr(ruleVo.getSort().equals(1) ? DroolsConstants.AND : ruleVo.getAndOr());
                strategyRule.setDeleted(DroolsConstants.DELETED_NO);
                strategyRule.setRuleId(ruleId);
                strategyRule.setSort(ruleVo.getSort());
                strategyRules.add(strategyRule);
            }
            // 插入 策略与规则的关联关系
            return strategyRuleService.insertBatch(strategyRules);
        }
        return false;
    }

    @Override
    public boolean copy(Long id) throws Exception {

        RecommendStrategy strategy = new RecommendStrategy();
        strategy.setId(id);
        strategy = strategy.selectById();
        if (strategy == null) {
            throw new Exception("该id不存在");
        }
        // 1. 复制策略
        String currentTimeStr = "-复制-".concat(String.valueOf(System.currentTimeMillis()));
        RecommendStrategy strategyTarget = new RecommendStrategy();
//        BeanUtils.copyProperties(strategyTarget, strategy);
        BeanUtils.copyProperties(strategy, strategyTarget);

        strategyTarget.setId(idGenerator.generate());
        strategyTarget.setName(strategyTarget.getName().concat(currentTimeStr));
        strategyTarget.setCreateById(null);
        strategyTarget.setCreateTime(null);
        strategyTarget.setCreateByName(null);
        strategyTarget.setUpdateById(null);
        strategyTarget.setUpdateByName(null);
        strategyTarget.setUpdateTime(null);
        ContextUtil.buildTenantEntity4Insert(strategyTarget);

        // 2. 复制相关的 rule 和 ruleItem
        RecommendStrategyRule strategyRule = new RecommendStrategyRule();
        strategyRule.setStrategyId(id);
        EntityWrapper<RecommendStrategyRule> ew = new EntityWrapper<>(strategyRule);
        List<RecommendStrategyRule> strategyRuleList = strategyRule.selectList(ew);

        List<RecommendStrategyRule> strategyRuleListTarget = new ArrayList<>();
        List<RecommendRule> ruleListTarget = new ArrayList<>();
        List<RecommendRuleItem> ruleItemListTarget = new ArrayList<>();

        if (!CollectionUtils.isEmpty(strategyRuleList)) {
            RecommendRule rule;
            RecommendRule ruleTarget;

            for (RecommendStrategyRule strategyRule4 : strategyRuleList) {
                rule = ruleService.selectById(strategyRule4.getRuleId());
                if (rule != null) {
                    // 2.1 如果不是模板的话
                    if (rule.getTemplate().equals(DroolsConstants.TEMPLATE_NO)) {
                        // 复制 rule
                        ruleTarget = copyRule(currentTimeStr, rule);
                        ruleListTarget.add(ruleTarget);
                        // 复制 rule_item
                        ruleItemListTarget.addAll(copyRuleItem(rule.getId(), ruleTarget.getId()));
                        // 复制一条关联关系
                        strategyRuleListTarget.add(copyStrategyRule(strategyTarget.getId(), strategyRule4, ruleTarget.getId()));
                    }
                    // 2.2 如果不是模板的话
                    else {
                        strategyRuleListTarget.add(copyStrategyRule(strategyTarget.getId(), strategyRule4, rule.getId()));
                    }
                }
            }
        }

        strategyTarget.insert();
        if (!CollectionUtils.isEmpty(ruleListTarget)) {
            ruleService.insertBatch(ruleListTarget);
        }
        if (!CollectionUtils.isEmpty(strategyRuleListTarget)) {
            strategyRuleService.insertBatch(strategyRuleListTarget);
        }
        if (!CollectionUtils.isEmpty(ruleItemListTarget)) {
            ruleItemService.insertBatch(ruleItemListTarget);
        }
        return true;
    }

    @Override
    public boolean delete(Long id) {
        int num = this.baseMapper.deleteById(id);
        if (num == 1) {
            EntityWrapper ew = new EntityWrapper();
            ew.eq("strategy_id", id);
            boolean flag = strategyRuleService.delete(ew);
            return flag;
        }
        return true;
    }

    @Override
    public Page<StrategyPageVo> pageByNameAndRuleId(String name, Long ruleId, Page page) {
        List<RecommendStrategy> list = this.baseMapper.listByNameAndRuleId(name, ruleId, new RowBounds(page.getOffset(), page.getLimit()));
        page.setRecords(list);
        page.setTotal(this.baseMapper.countListByNameAndRuleId(name, ruleId));
        return page;
    }

    private RecommendStrategyRule copyStrategyRule(Long strategyId, RecommendStrategyRule origin, Long ruleId) {
        RecommendStrategyRule sr = new RecommendStrategyRule();
        sr.setStrategyId(strategyId);
        sr.setSort(origin.getSort());
        sr.setRuleId(ruleId);
        sr.setDeleted(DroolsConstants.DELETED_NO);
        sr.setAndOr(origin.getAndOr());
        sr.setId(idGenerator.generate());
        return sr;
    }

    private RecommendRule copyRule(String currentTimeStr, RecommendRule origin) {
        RecommendRule ruleTarget = new RecommendRule();
        ruleTarget.setType(origin.getType());
        ruleTarget.setDeleted(DroolsConstants.DELETED_NO);
        ruleTarget.setId(idGenerator.generate());
        ruleTarget.setTemplate(origin.getTemplate());
        ruleTarget.setName(origin.getName().concat(currentTimeStr));
        ContextUtil.buildTenantEntity4Insert(ruleTarget);
        return ruleTarget;
    }

    private List<RecommendRuleItem> copyRuleItem(Long ruleId, Long ruleIdTarget) {
        RecommendRuleItem ruleItem = new RecommendRuleItem();
        ruleItem.setRuleId(ruleId);
        ContextUtil.buildTenantEntity4Select(ruleItem);
        List<RecommendRuleItem> ruleItemList = ruleItem.selectList(new EntityWrapper(ruleItem));
        List<RecommendRuleItem> ruleItemListTarget = new ArrayList<>();
        if (!CollectionUtils.isEmpty(ruleItemList)) {
            for (RecommendRuleItem item : ruleItemList) {
                RecommendRuleItem ruleItemTarget = new RecommendRuleItem();
                ruleItemTarget.setId(idGenerator.generate());
                ruleItemTarget.setBusiness(item.getBusiness());
                ruleItemTarget.setName(item.getName());
                ruleItemTarget.setRuleId(ruleIdTarget);
                ruleItemTarget.setAndOr(item.getAndOr());
                ruleItemTarget.setClassName(item.getClassName());
                ruleItemTarget.setDeleted(DroolsConstants.DELETED_NO);
                ruleItemTarget.setPropertyName(item.getPropertyName());
                ruleItemTarget.setRelation(item.getRelation());
                ruleItemTarget.setSort(item.getSort());
                ruleItemTarget.setTargetVal(item.getTargetVal());
                ruleItemTarget.setType(item.getType());
                ContextUtil.buildTenantEntity4Insert(ruleItemTarget);
                ruleItemListTarget.add(ruleItemTarget);
            }
        }
        return ruleItemListTarget;
    }

    @Override
    public Page getVisibleRange(Long id, String keyword, Integer pageNo, Integer pageSize) {
        Page page = new Page(pageNo, pageSize);
        Map<Long, String> map = new HashMap();
        List<Long> ruleIds = null;
        List<RecommendStrategyRule> recommendStrategyRules = this.strategyRuleService.listByStrategyId(id);
        if (!CollectionUtils.isEmpty(recommendStrategyRules)) {
            ruleIds = new ArrayList<>(recommendStrategyRules.size());
            for (RecommendStrategyRule a : recommendStrategyRules) {
                if (!map.containsKey(a.getRuleId())) {
                    map.put(a.getRuleId(), a.getAndOr());
                }
                ruleIds.add(a.getRuleId());
            }
        }
        if (!CollectionUtils.isEmpty(ruleIds)) {

            RequestContext context = ContextHolder.get();
            RuleVisibleRangeVo rangeVo = new RuleVisibleRangeVo();
            rangeVo.setKeyword(keyword);
            rangeVo.setPageNo(pageNo);
            rangeVo.setPageSize(pageSize);
            rangeVo.setCompanyId(context.getCompanyId());
            rangeVo.setSiteId(context.getSiteId());
            rangeVo.setIsStrategy(true);

            List<SystemRuleItemVo> ruleItemVos = new ArrayList<>();
            for (Long ruleId : ruleIds) {
                List<RecommendRuleItem> items = ruleItemService.listByRuleId(ruleId, null);
                if (!CollectionUtils.isEmpty(items)) {

                    items.forEach(a -> {
                        //只获取用户规则的项
                        if (a.getType() == 2) {
                            SystemRuleItemVo vo = new SystemRuleItemVo();
                            BeanUtils.copyProperties(a, vo);
                            vo.setId(a.getId());
                            vo.setRulesAndOr(map.get(a.getRuleId()));
                            ruleItemVos.add(vo);
                        }
                    });
                }
            }
            rangeVo.setRuleItemVos(ruleItemVos);
            PageInfo<DroolsAccountVO> pageByDrools = accountClient.getPageByRuleItems(rangeVo);
            page.setRecords(pageByDrools.getRecords());
            page.setTotal(pageByDrools.getPageRecords());
        }
        return page;
    }
}